/*
 * Copyright (c) 2023 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: Apache-2.0
 */
#include <zephyr/kernel.h>
#include <zephyr/sys/util.h>
#include <zephyr/mgmt/mcumgr/mgmt/mgmt.h>
#include <zephyr/mgmt/mcumgr/smp/smp.h>
#include <zephyr/mgmt/mcumgr/mgmt/handlers.h>
/* The below should be updated with the real name of the file */
#include <example_mgmt.h>
//#include <zephyr/logging/log.h>
#include <assert.h>
#include <limits.h>
#include <string.h>
#include <stdio.h>
#include <zcbor_common.h>
#include <zcbor_decode.h>
#include <zcbor_encode.h>
#include <mgmt/mcumgr/util/zcbor_bulk.h>

K_MUTEX_DEFINE(beacons_mutex);

//LOG_MODULE_REGISTER(mcumgr_example_grp, CONFIG_MCUMGR_GRP_EXAMPLE_LOG_LEVEL);
/* Example function with "read" command support, requires both parameters are supplied */
#if 0
static int example_mgmt_test(struct smp_streamer *ctxt)
{
	uint32_t uint_value = 0;
	zcbor_state_t *zse = ctxt->writer->zs;
	zcbor_state_t *zsd = ctxt->reader->zs;
	bool ok;
	struct zcbor_string string_value = { 0 };
	size_t decoded;
	struct zcbor_map_decode_key_val example_test_decode[] = {
		ZCBOR_MAP_DECODE_KEY_DECODER("uint_key", zcbor_uint32_decode, &uint_value),
		ZCBOR_MAP_DECODE_KEY_DECODER("string_key", zcbor_tstr_decode, &string_value),
	};

	printk("Example test function called\n");

	ok = zcbor_map_decode_bulk(zsd, example_test_decode, ARRAY_SIZE(example_test_decode),
				   &decoded) == 0;
	/* Check that both parameters were supplied and that the value of "string_key" is not
	 * empty
	 */
	if (!ok || string_value.len == 0 || !zcbor_map_decode_bulk_key_found(
			   example_test_decode, ARRAY_SIZE(example_test_decode), "uint_key")) {
		return MGMT_ERR_EINVAL;
	}

	/* If the value of "uint_key" is over 50, return an error of "not wanted" */
	if (uint_value > 50) {
		ok = smp_add_cmd_err(zse, MGMT_GROUP_ID_EXAMPLE, EXAMPLE_MGMT_ERR_NOT_WANTED);
		goto end;
	}

	/* Otherwise, return an integer value of 4691 */
	ok = zcbor_tstr_put_lit(zse, "return_int") &&
	     zcbor_int32_put(zse, 4691);

end:
	/* If "ok" is false, then there was an error processing the output cbor message, which
	 * likely indicates a lack of available memory
	 */
	return (ok ? MGMT_ERR_EOK : MGMT_ERR_EMSGSIZE);
}
#endif

#define BLE_MGMT_MAX_BEACONS_REPORTS    5
static ble_beacon_t ble_beacons[BLE_MGMT_MAX_BEACONS_REPORTS];

static int example_mgmt_test(struct smp_streamer *ctxt)
{
	uint32_t uint_value = 0;
	zcbor_state_t *zse = ctxt->writer->zs;
	zcbor_state_t *zsd = ctxt->reader->zs;
	bool ok;

#if 0
	struct zcbor_string string_value = { 0 };
	size_t decoded;
	struct zcbor_map_decode_key_val example_test_decode[] = {
		ZCBOR_MAP_DECODE_KEY_DECODER("uint_key", zcbor_uint32_decode, &uint_value),
		ZCBOR_MAP_DECODE_KEY_DECODER("string_key", zcbor_tstr_decode, &string_value),
	};

	printk("Example test function called\n");

	ok = zcbor_map_decode_bulk(zsd, example_test_decode, ARRAY_SIZE(example_test_decode),
				   &decoded) == 0;
	/* Check that both parameters were supplied and that the value of "string_key" is not
	 * empty
	 */
	if (!ok || string_value.len == 0 || !zcbor_map_decode_bulk_key_found(
			   example_test_decode, ARRAY_SIZE(example_test_decode), "uint_key")) {
		return MGMT_ERR_EINVAL;
	}

	/* If the value of "uint_key" is over 50, return an error of "not wanted" */
	if (uint_value > 50) {
		ok = smp_add_cmd_err(zse, MGMT_GROUP_ID_EXAMPLE, EXAMPLE_MGMT_ERR_NOT_WANTED);
		goto end;
	}
#endif
	// return BLE beacons info
	ble_beacon_t beacon;

	printk("Beacon data handler\n");

	// Get beacon data
	k_mutex_lock(&beacons_mutex, K_FOREVER);

	// Copy the data
	memcpy(&beacon, ble_beacons, sizeof(ble_beacon_t));
	
	ble_beacons[0].type = BLE_DEV_ADDR_TYPE_NONE;

	k_mutex_unlock(&beacons_mutex);

	if (beacon.type != BLE_DEV_ADDR_TYPE_NONE)
	{
		// 1 sec invalidation timeout
		// if (beacon.timestamp + 1000 > k_uptime_get())
		// {
		// 	// Invalidate the data
		// 	beacon.type = BLE_DEV_ADDR_TYPE_NONE;
		// }
		// else
		//{
			printk("Beacon data: %d\n", beacon.type);

			const struct zcbor_string input = {
				.value = (const uint8_t *)&beacon.mac_addr[0],
				.len = 6,
			};

			/* Otherwise, return an integer value of 4691 */
			// ok = zcbor_tstr_put_lit(zse, "beacons_data") &&
			// 	zcbor_bstr_encode(zse, &input);

			ok = zcbor_tstr_put_lit(zse, "addr_type") && zcbor_uint32_put(zse, 1) &&
				zcbor_tstr_put_lit(zse, "mac") && zcbor_bstr_encode(zse, &input) &&
				zcbor_tstr_put_lit(zse, "data_len") && zcbor_uint32_put(zse, beacon.data_len) &&
				zcbor_tstr_put_lit(zse, "rssi") && zcbor_int32_put(zse, beacon.rssi) &&
				zcbor_tstr_put_lit(zse, "adv_type") && zcbor_uint32_put(zse, beacon.adv_type) &&
				zcbor_tstr_put_lit(zse, "last_heard") && zcbor_int64_put(zse, sys_clock_tick_get());
					//k_uptime_seconds());
				// sys_clock_tick_get_32()


			beacon.type = BLE_DEV_ADDR_TYPE_NONE;

		//}
	}
	else
	{
		printk("No beacon data\n");
		
		ok = zcbor_map_start_encode(zse, 0) &&
		zcbor_map_end_encode(zse, 0);
	}

//end:
	/* If "ok" is false, then there was an error processing the output cbor message, which
	 * likely indicates a lack of available memory
	 */
	return (ok ? MGMT_ERR_EOK : MGMT_ERR_EMSGSIZE);
}

/* Example function with "write" command support */
static int example_mgmt_other(struct smp_streamer *ctxt)
{
	uint32_t user_value = 0;
	zcbor_state_t *zse = ctxt->writer->zs;
	zcbor_state_t *zsd = ctxt->reader->zs;
	bool ok;
	size_t decoded;
	struct zcbor_map_decode_key_val example_other_decode[] = {
		ZCBOR_MAP_DECODE_KEY_DECODER("user_value", zcbor_uint32_decode, &user_value),
	};

	printk("Example other function called\n");

	ok = zcbor_map_decode_bulk(zsd, example_other_decode, ARRAY_SIZE(example_other_decode),
				   &decoded) == 0;

	/* The supplied value is optional, therefore do not return an error if it was not
	 * provided
	 */
	if (!ok) {
		return MGMT_ERR_EINVAL;
	}

	/* Return some dummy data to the client */
	ok = zcbor_tstr_put_lit(zse, "return_string") &&
	     zcbor_tstr_put_lit(zse, "some dummy data!");

	/* If "ok" is false, then there was an error processing the output cbor message, which
	 * likely indicates a lack of available memory
	 */
	return (ok ? MGMT_ERR_EOK : MGMT_ERR_EMSGSIZE);
}

#ifdef CONFIG_MCUMGR_SMP_SUPPORT_ORIGINAL_PROTOCOL
/* This is a lookup function that converts from SMP version 2 group error codes to legacy
 * MCUmgr error codes, it is only included if support for the original protocol is enabled.
 * Note that in SMP version 2, MCUmgr error codes can still be returned, but are to be used
 * only for general SMP/MCUmgr errors. The success/OK error code is not used in translation
 * functions as it is automatically handled by the base SMP code.
 */
static int example_mgmt_translate_error_code(uint16_t err)
{
	int rc;

	switch (err) {
	case EXAMPLE_MGMT_ERR_NOT_WANTED:
		rc = MGMT_ERR_ENOENT;
		break;

	case EXAMPLE_MGMT_ERR_REJECTED_BY_HOOK:
		rc = MGMT_ERR_EBADSTATE;
		break;

	case EXAMPLE_MGMT_ERR_UNKNOWN:
	default:
		rc = MGMT_ERR_EUNKNOWN;
	}

	return rc;
}
#endif

static const struct mgmt_handler example_mgmt_handlers[] = {
	[EXAMPLE_MGMT_ID_TEST] = {
		.mh_read = example_mgmt_test,
		.mh_write = NULL,
	},
	[EXAMPLE_MGMT_ID_OTHER] = {
		.mh_read = NULL,
		.mh_write = example_mgmt_other,
	},
};

static struct mgmt_group example_mgmt_group = {
	.mg_handlers = example_mgmt_handlers,
	.mg_handlers_count = ARRAY_SIZE(example_mgmt_handlers),
	.mg_group_id = MGMT_GROUP_ID_EXAMPLE,
#ifdef CONFIG_MCUMGR_SMP_SUPPORT_ORIGINAL_PROTOCOL
	.mg_translate_error = example_mgmt_translate_error_code,
#endif
};

static void example_mgmt_register_group(void)
{
	/* This function is called during system init before main() is invoked, if the
	 * handler needs to set anything up before it can be used, it should do it here.
	 * This register the group so that clients can call the function handlers.
	 */
	// Invalidate the beacons data
	for (size_t i = 0; i < ARRAY_SIZE(ble_beacons); i++)
	{
		ble_beacons[i].type = BLE_DEV_ADDR_TYPE_NONE;
	}

	mgmt_register_group(&example_mgmt_group);
}

void beacons_update(ble_beacon_t beacon)
{
	k_mutex_lock(&beacons_mutex, K_FOREVER);

	// Copy the data
	memcpy(ble_beacons, &beacon, sizeof(ble_beacon_t));
	// Update time
	//beacons_data->timestamp = k_uptime_get();

	k_mutex_unlock(&beacons_mutex);
}

MCUMGR_HANDLER_DEFINE(example_mgmt, example_mgmt_register_group);
